@@ -1,6 +1,6 @@
 """
 Ludolph: Monitoring Jabber Bot
-Copyright (C) 2012-13 Erigones s.r.o.
+Copyright (C) 2012-2013 Erigones s. r. o.
 This file is part of Ludolph.
 
 See the LICENSE file for copying permission.
@@ -9,25 +9,41 @@ from logging import getLogger
 
 logger = getLogger(__name__)
 
-COMMANDS = {} # command : {name, module, doc}
-USERS = [] # List of users
-ADMINS = [] # List of admins
+COMMANDS = {}  # command : {name, module, doc}
+USERS = set()  # List of users
+ADMINS = set() # List of admins
+
+__all__ = ['command', 'parameter_required', 'admin_required']
+
 
 def command(f):
     """
     Decorator for registering available commands.
     """
+    global COMMANDS
+    global USERS
+
     def wrap(obj, msg, *args, **kwargs):
-        if not USERS or msg['from'].bare in USERS:
-            logger.info('User "%s" requested command "%s"' % (msg['from'],msg['body']))
-            # Reply with output of function
+        user = obj.xmpp.get_jid(msg)
+        cmd = '%s.%s' % (f.__module__, f.__name__)
+
+        if not USERS or user in USERS:
+            logger.info('User "%s" requested command "%s" (%s)', user, msg['body'], cmd)
+            # Parse optional parameters
+            if f.func_defaults:
+                required_pos = len(args) + 1
+                optional_end = required_pos + len(f.func_defaults)
+                optional_args = msg['body'].strip().split()[required_pos:optional_end]
+                args += tuple(optional_args)
+
+            # Reply with function output
             out = f(obj, msg, *args, **kwargs)
             logger.debug('Command output: "%s"', out)
-            return msg.reply(out).send()
+            obj.xmpp.msg_reply(msg, out)
+            return True
         else:
-            logger.warning('Unauthorized command "%s" from "%s"' % (
-                msg['body'], msg['from']))
-            msg.reply('Permission denied').send()
+            logger.warning('Unauthorized command "%s" (%s) from "%s"', msg['body'], cmd, user)
+            obj.xmpp.msg_reply(msg, 'ERROR: Permission denied')
             return None
 
     # Create command name - skip methods which start with underscore
@@ -39,13 +55,12 @@ def command(f):
 
     # Check if command exists
     if name in COMMANDS.keys():
-        logger.critical('Command "%s" from plugin "%s" overlaps with existing '
-                'command from module "%s"' % (name, f.__module__,
-                    COMMANDS[name]['module']))
+        logger.critical('Command "%s" from plugin "%s" overlaps with existing command from module "%s"',
+                        name, f.__module__, COMMANDS[name]['module'])
         return None
 
     # Save module and method name
-    COMMANDS[name] = { 'name': f.__name__, 'module': f.__module__ }
+    COMMANDS[name] = {'name': f.__name__, 'module': f.__module__}
 
     # Save documentation
     if f.__doc__:
@@ -56,6 +71,7 @@ def command(f):
 
     return wrap
 
+
 def parameter_required(count=1):
     """
     Decorator for required command parameters.
@@ -64,29 +80,34 @@ def parameter_required(count=1):
         def wrap(obj, msg, *args, **kwargs):
             #Try to get command parameter
             params = msg['body'].strip().split()[1:]
-            if len(params) == count:
+
+            if len(params) < count:
+                user = obj.xmpp.get_jid(msg)
+                logger.warning('Missing parameter in command "%s" from user "%s"', msg['body'], user)
+                obj.xmpp.msg_reply(msg, 'ERROR: Missing parameter')
+                return None
+            else:
                 params.extend(args)
                 return f(obj, msg, *params, **kwargs)
-            else:
-                logger.warning('Missing parameter in command "%s" from user "%s"' % (
-                    msg['body'], msg['from']))
-                msg.reply('Missing parameter').send()
-                return None
 
         return wrap
     return parameter_required_decorator
 
+
 def admin_required(f):
     """
     Decorator for admin only commands.
     """
+    global ADMINS
+
     def wrap(obj, msg, *args, **kwargs):
-        if not ADMINS or msg['from'].bare in ADMINS:
+        user = obj.xmpp.get_jid(msg)
+
+        if not ADMINS or user in ADMINS:
             return f(obj, msg, *args, **kwargs)
         else:
-            logger.warning('Unauthorized command "%s" from user "%s"' % (
-                msg['body'], msg['from']))
-            msg.reply('Permission denied').send()
+            logger.warning('Unauthorized command "%s" from user "%s"', msg['body'], user)
+            obj.xmpp.msg_reply(msg, 'ERROR: Permission denied')
             return None
 
     return wrap
